'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var backendCommon = require('@backstage/backend-common');
var express = require('express');
var Router = require('express-promise-router');
var utilArnParser = require('@aws-sdk/util-arn-parser');
var awsCorePluginForBackstageCommon = require('@aws/aws-core-plugin-for-backstage-common');
var catalogModel = require('@backstage/catalog-model');
var integrationAwsNode = require('@backstage/integration-aws-node');
var clientCodebuild = require('@aws-sdk/client-codebuild');
var awsCodebuildPluginForBackstageCommon = require('@aws/aws-codebuild-plugin-for-backstage-common');
var backendPluginApi = require('@backstage/backend-plugin-api');
var alpha = require('@backstage/plugin-catalog-node/alpha');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e : { default: e }; }

var express__default = /*#__PURE__*/_interopDefaultCompat(express);
var Router__default = /*#__PURE__*/_interopDefaultCompat(Router);

class DefaultAwsCodeBuildService {
  constructor(logger, auth, catalogApi, resourceLocator, credsManager) {
    this.logger = logger;
    this.auth = auth;
    this.catalogApi = catalogApi;
    this.resourceLocator = resourceLocator;
    this.credsManager = credsManager;
  }
  static async fromConfig(config, options) {
    var _a;
    const credsManager = integrationAwsNode.DefaultAwsCredentialsManager.fromConfig(config);
    const { auth } = backendCommon.createLegacyAuthAdapters(options);
    const resourceLocator = (_a = options == null ? void 0 : options.resourceLocator) != null ? _a : await awsCorePluginForBackstageCommon.AwsResourceLocatorFactory.fromConfig(config, options.logger);
    return new DefaultAwsCodeBuildService(
      options.logger,
      auth,
      options.catalogApi,
      resourceLocator,
      credsManager
    );
  }
  async getProjectsByEntity(options) {
    var _a;
    (_a = this.logger) == null ? void 0 : _a.debug(`Fetch CodeBuild projects for ${options.entityRef}`);
    const arns = await this.getCodeBuildArnsForEntity(options);
    const projects = await Promise.all(
      arns.map(async (arn) => {
        var _a2;
        const { region, accountId, resource } = utilArnParser.parse(arn);
        const projectName = resource.split("/")[1];
        const client = await this.getClient(region, arn);
        const projectResponse = await client.send(
          new clientCodebuild.BatchGetProjectsCommand({
            names: [projectName]
          })
        );
        const project = projectResponse.projects[0];
        const buildIds = await client.send(
          new clientCodebuild.ListBuildsForProjectCommand({
            projectName
          })
        );
        let builds = [];
        if (buildIds.ids && buildIds.ids.length > 0) {
          const output = await client.send(
            new clientCodebuild.BatchGetBuildsCommand({
              ids: buildIds.ids.slice(0, 5)
            })
          );
          builds = (_a2 = output.builds) != null ? _a2 : [];
        }
        return {
          project,
          projectName,
          projectRegion: region,
          projectAccountId: accountId,
          builds
        };
      })
    );
    return {
      projects
    };
  }
  async getCodeBuildArnsForEntity(options) {
    const entity = await this.catalogApi.getEntityByRef(
      options.entityRef,
      options.credentials && await this.auth.getPluginRequestToken({
        onBehalfOf: options.credentials,
        targetPluginId: "catalog"
      })
    );
    if (!entity) {
      throw new Error(
        `Couldn't find entity with name: ${catalogModel.stringifyEntityRef(
          options.entityRef
        )}`
      );
    }
    const annotation = awsCorePluginForBackstageCommon.getOneOfEntityAnnotations(entity, [
      awsCodebuildPluginForBackstageCommon.AWS_CODEBUILD_ARN_ANNOTATION,
      awsCodebuildPluginForBackstageCommon.AWS_CODEBUILD_TAGS_ANNOTATION,
      awsCodebuildPluginForBackstageCommon.AWS_CODEBUILD_ARN_ANNOTATION_LEGACY
    ]);
    if (!annotation) {
      throw new Error("Annotation not found on entity");
    }
    let arns;
    if (annotation.name === awsCodebuildPluginForBackstageCommon.AWS_CODEBUILD_TAGS_ANNOTATION) {
      arns = await this.resourceLocator.getResourceArns({
        resourceType: "AWS::CodeBuild::Project",
        tagString: annotation.value
      });
    } else {
      arns = [annotation.value];
    }
    return Promise.resolve(arns);
  }
  async getClient(region, arn) {
    const credentialProvider = await this.credsManager.getCredentialProvider({
      arn
    });
    return new clientCodebuild.CodeBuildClient({
      region,
      customUserAgent: awsCorePluginForBackstageCommon.AWS_SDK_CUSTOM_USER_AGENT,
      credentialDefaultProvider: () => credentialProvider.sdkCredentialProvider
    });
  }
}

async function createRouter(options) {
  const { logger, awsCodeBuildApi } = options;
  const router = Router__default.default();
  router.use(express__default.default.json());
  const { httpAuth } = backendCommon.createLegacyAuthAdapters(options);
  router.get(
    "/v1/entity/:namespace/:kind/:name/projects",
    async (request, response) => {
      const { namespace, kind, name } = request.params;
      const services = await awsCodeBuildApi.getProjectsByEntity({
        entityRef: {
          kind,
          namespace,
          name
        },
        credentials: await httpAuth.credentials(request)
      });
      response.status(200).json(services);
    }
  );
  router.get("/health", (_, response) => {
    logger.info("PONG!");
    response.json({ status: "ok" });
  });
  router.use(backendCommon.errorHandler());
  return router;
}

const awsCodebuildPlugin = backendPluginApi.createBackendPlugin({
  pluginId: "aws-codebuild",
  register(env) {
    env.registerInit({
      deps: {
        logger: backendPluginApi.coreServices.logger,
        httpRouter: backendPluginApi.coreServices.httpRouter,
        config: backendPluginApi.coreServices.rootConfig,
        catalogApi: alpha.catalogServiceRef,
        auth: backendPluginApi.coreServices.auth,
        discovery: backendPluginApi.coreServices.discovery,
        httpAuth: backendPluginApi.coreServices.httpAuth
      },
      async init({
        logger,
        httpRouter,
        config,
        catalogApi,
        auth,
        httpAuth,
        discovery
      }) {
        const winstonLogger = backendCommon.loggerToWinstonLogger(logger);
        const awsCodeBuildApi = await DefaultAwsCodeBuildService.fromConfig(
          config,
          {
            catalogApi,
            auth,
            httpAuth,
            discovery,
            logger: winstonLogger
          }
        );
        httpRouter.use(
          await createRouter({
            logger: winstonLogger,
            awsCodeBuildApi,
            discovery,
            auth,
            httpAuth
          })
        );
        httpRouter.addAuthPolicy({
          path: "/health",
          allow: "unauthenticated"
        });
      }
    });
  }
});

exports.DefaultAwsCodeBuildService = DefaultAwsCodeBuildService;
exports.createRouter = createRouter;
exports.default = awsCodebuildPlugin;
//# sourceMappingURL=index.cjs.js.map
